import collections
import pydoc

from django.utils.translation import ugettext_lazy as _

from django_vox import settings

PROTOCOLS = {
    'email': _('Email'),
    'sms': _('SMS'),
    'slack-webhook': _('Slack Webhook'),
}


class BackendManager:

    def __init__(self, class_list):
        self.proto_map = collections.defaultdict(list)
        self.id_map = {}
        for cls in class_list:
            if cls.ID in self.id_map:
                raise RuntimeError(
                    'Conflicting backend IDs: {}'.format(cls.ID))
            self.proto_map[cls.PROTOCOL].append(cls)
            self.id_map[cls.ID] = cls

    def by_protocol(self, protocol: str):
        return self.proto_map[protocol]

    def by_id(self, id_val):
        return self.id_map[id_val]

    def all(self):
        return self.id_map.values()

    def protocols(self):
        return self.proto_map.keys()


BACKENDS = BackendManager(
    pydoc.locate(name) for name in settings.BACKENDS)


def get_protocol_choices():
    for protocol in BACKENDS.protocols():
        yield (protocol, PROTOCOLS.get(protocol, protocol))



# -- Django workaround
import django
import os
import sys
sys.path.append(os.path.dirname(__file__))
os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
django.setup()

# -- Project information -----------------------------------------------------

project = 'Django Vox'
copyright = '2018, Alan Trick'
author = 'Alan Trick'

extensions = [
    'sphinx.ext.doctest',
    'sphinx.ext.autodoc',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

source_suffix = '.rst'

master_doc = 'index'

language = None

exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

pygments_style = 'sphinx'


# -- Options for HTML output -------------------------------------------------

html_theme = 'classic'
html_static_path = ['_static']
htmlhelp_basename = 'DjangoVoxdoc'

# -- Options for LaTeX output ------------------------------------------------

latex_elements = {}

latex_documents = [
    (master_doc, 'DjangoVox.tex', 'Django Vox Documentation',
     'Alan Trick', 'manual'),
]


# -- Options for manual page output ------------------------------------------

man_pages = [
    (master_doc, 'djangovox', 'Django Vox Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

texinfo_documents = [
    (master_doc, 'DjangoVox', 'Django Vox Documentation',
     author, 'DjangoVox', 'One line description of project.',
     'Miscellaneous'),
]



import os

from django.apps import AppConfig
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

import tests.models
from django_vox.management.commands import make_notifications
from django_vox.models import Notification


class MakeNotificationTests(TestCase):

    @staticmethod
    def test_make_notifications():
        cmd = make_notifications.Command()
        assert Notification.objects.all().count() == 0
        # dry run
        cmd.handle(dry_run=True)
        assert Notification.objects.all().count() == 0
        # test basic notification making
        cmd.handle(verbose=True)
        assert Notification.objects.all().count() == 3
        # gather some general things
        article_ct = ContentType.objects.get_for_model(
            tests.models.Article)
        user_ct = ContentType.objects.get_for_model(
            tests.models.User)

        # make a notification to delete
        Notification.objects.create(
            codename='foo',
            content_type=article_ct,
            from_code=True,
        )
        assert Notification.objects.all().count() == 4
        cmd.handle()
        assert Notification.objects.all().count() == 3
        # here's one that shouldn't get deleted
        Notification.objects.create(
            codename='foo',
            content_type=article_ct,
            from_code=False,
        )
        assert Notification.objects.all().count() == 4
        cmd.handle()
        assert Notification.objects.all().count() == 4
        # now we'll change one and it should get reverted
        acn = Notification.objects.get_by_natural_key(
            'tests', 'article', 'created')
        acn.source_model = user_ct
        acn.save()
        cmd.handle(verbose=True)
        acn = Notification.objects.get_by_natural_key(
            'tests', 'article', 'created')
        assert acn.source_model is None

    @staticmethod
    def test_bad_appconfig():
        config = AppConfig('os', os)
        make_notifications.make_notifications(config)



import importlib.util
import pydoc
import re

import django.conf

URL_PATTERN = re.compile(
    r'((([A-Za-z]{3,9}:(?:\/\/)?)(?:[\-;:&=\+\$,\w]+@)?[A-Za-z0-9\.\-]'
    r'+(:[0-9]+)?|(?:www\.|[\-;:&=\+\$,\w]+@)[A-Za-z0-9\.\-]+)((?:\/[\+'
    r'~%\/\.\w\-_]*)?\??(?:[\-\+=&;%@\.\w_]*)#?(?:[\.\!\/\\\w]*))?)')

BACKENDS = getattr(django.conf.settings, 'DJANGO_VOX_BACKENDS', None)
if BACKENDS is None:
    BACKENDS = []
    # Automatically set based on libraries available
    default_backends = [
        'django_vox.backends.html_email.Backend',
        'django_vox.backends.markdown_email.Backend',
        'django_vox.backends.postmark_email.Backend',
        'django_vox.backends.template_email.Backend',
        'django_vox.backends.twilio.Backend',
        'django_vox.backends.slack.Backend',
        'django_vox.backends.json_webhook.Backend',
    ]
    for cls_str in default_backends:
        cls = pydoc.locate(cls_str)
        for dep in cls.DEPENDS:
            if importlib.util.find_spec(dep) is None:
                continue
        BACKENDS.append(cls_str)

MARKDOWN_EXTRAS = getattr(django.conf.settings,
                          'DJANGO_VOX_MARKDOWN_EXTRAS', None)
if MARKDOWN_EXTRAS is None:
    MARKDOWN_EXTRAS = ['footnotes', 'link_patterns', 'smarty-pants', 'tables']

MARKDOWN_LINK_PATTERNS = getattr(django.conf.settings,
                                 'DJANGO_VOX_MARKDOWN_LINK_PATTERNS', None)
if MARKDOWN_LINK_PATTERNS is None:
    MARKDOWN_LINK_PATTERNS = [(URL_PATTERN, r'\1')]

SENDER_MODEL = getattr(django.conf.settings,
                       'DJANGO_VOX_SENDER_MODEL', None)
RECIPIENT_MODEL = getattr(django.conf.settings,
                          'DJANGO_VOX_RECIPIENT_MODEL', None)



import django.conf
import django.core.mail.backends.base
import django.core.mail.backends.smtp
import django.template
import django.utils.html
from django.utils.translation import ugettext_lazy as _

from . import base

__all__ = ('Backend',)


class Backend(base.Backend):

    ID = 'twilio'
    PROTOCOL = 'sms'
    ESCAPE_HTML = False
    VERBOSE_NAME = _('Twilio')
    DEPENDS = ('twilio',)

    @classmethod
    def send_message(cls, contact, message):
        from twilio.rest import Client
        if not hasattr(django.conf.settings, 'TWILIO_ACCOUNT_SID'):
            raise django.conf.ImproperlyConfigured(
                'Twilio backend enabled but TWILIO_* settings missing')
        account_sid = django.conf.settings.TWILIO_ACCOUNT_SID
        auth_token = django.conf.settings.TWILIO_AUTH_TOKEN
        from_number = django.conf.settings.TWILIO_FROM_NUMBER
        client = Client(account_sid, auth_token)
        client.messages.create(
            to=contact.address, from_=from_number, body=message)



import django.conf
import django.core.mail.backends.base
import django.core.mail.backends.smtp
import django.template
import django.utils.html
from django.template import Context
from django.utils.translation import ugettext_lazy as _

__ALL__ = ('PROTOCOLS', 'Backend', 'template_from_string')

PROTOCOLS = {
    'email': _('Email'),
    'sms': _('SMS'),
    'webhook': _('Webhook'),
    'slack-webhook': _('Slack Webhook'),
}


def html_format(text: str):
    escaped = django.utils.html.escape(text)
    return escaped.replace('\r\n', '<br/>').replace('\n', '<br/>')


class Backend:

    USE_SUBJECT = False
    ESCAPE_HTML = True
    DEPENDS = ()

    @classmethod
    def build_message(cls, _subject: str, body: str, parameters: dict):
        template = template_from_string(body)
        context = Context(parameters, autoescape=cls.ESCAPE_HTML)
        return template.render(context)

    @classmethod
    def preview_message(cls, subject: str, body: str, parameters: dict):
        message = cls.build_message(subject, body, parameters)
        if not cls.ESCAPE_HTML:
            message = html_format(message)
        return message

    @classmethod
    def send_message(
            cls, contact: 'django_vox.base.Contact', message):
        raise NotImplementedError


def template_from_string(text: str, using=None) -> \
        django.template.base.Template:
    """
    Convert a string into a template object,
    using a given template engine or using the default backends
    from settings.TEMPLATES if no engine was specified.
    """
    # This function is based on django.template.loader.get_template,
    # but uses Engine.from_string instead of Engine.get_template.
    engines = django.template.engines
    engine_list = engines.all() if using is None else [engines[using]]
    exception = None
    for engine in engine_list:
        try:
            return engine.from_string(text).template
        except django.template.exceptions.TemplateSyntaxError as e:
            exception = e
    raise exception



import typing

import django.conf
import django.core.mail.backends.base
import django.core.mail.backends.smtp
import django.template
import django.utils.html
import requests
from django.utils.html import escape
from django.utils.translation import ugettext_lazy as _

from . import base

__all__ = ('Backend',)


class Backend(base.Backend):

    ID = 'json-webhook'
    PROTOCOL = 'json-webhook'
    VERBOSE_NAME = _('Generic Webhook (JSON)')
    DEPENDS = ('lxml',)

    @classmethod
    def parse_message(cls, body: str) -> typing.Mapping[str, str]:
        data = {}
        for line in body.split('\n'):
            parts = line.split(':')
            key = parts[0].strip()
            if key == '':
                continue
            data[key] = (':'.join(parts[1:])).strip()
        return data

    @classmethod
    def build_message(cls, subject: str, body: str, parameters: dict):
        data = cls.parse_message(body)
        def_list = '\n'.join(
            '<dt>{}</dt><dd>{}</dd>'.format(escape(key), escape(value))
            for key, value in data.items()
        )
        html = '<html>\n<h1>{}</h1>\n<dl>\n{}\n</dl>\n</html>'.format(
            escape(subject), def_list)
        context = django.template.Context(parameters)
        template = base.template_from_string(html)
        return template.render(context)

    @classmethod
    def preview_message(cls, subject: str, body: str, parameters: dict):
        return cls.build_message(subject, body, parameters)

    @classmethod
    def extract_model(cls, message):
        from lxml import etree
        tree = etree.fromstring(message)
        subject = tree[0].text
        model = {}
        key = ''
        for element in tree[1]:
            if element.tag == 'dt':
                key = element.text
            else:
                model[key] = element.text
        return subject, model

    @classmethod
    def send_message(cls, contact, message):
        subject, model = cls.extract_model(message)
        headers = {'Content-Type': 'application/json'}
        response = requests.post(contact.address, json=model, headers=headers)
        if not response.ok:
            raise RuntimeError(
                'HTTP error: {} {}'.format(response.text))



from django.apps import apps
from django.contrib.contenttypes.models import ContentType

import django_vox.models


class BackgroundVoxModel(django_vox.models.VoxModel):
    """
    A vox model that sends its calls though a background task
    """

    class Meta:
        abstract = True

    def issue_notification(self, codename: str,
                           target: django_vox.models.VoxModelN = None,
                           source: django_vox.models.VoxModelN = None):
        kwargs = {}
        self_cls_str = str(self.__class__._meta)
        if target is not None:
            kwargs['target_cls_str'] = str(target.__class__._meta)
            kwargs['target_id'] = target.id
        if source is not None:
            kwargs['source_cls_str'] = str(source.__class__._meta)
            kwargs['source_id'] = source.id
        issue_notification(codename, self_cls_str, self.id, **kwargs)


def issue_notification(
        codename: str,
        content_cls_str: str,
        content_id: int,
        target_cls_str: str = '',
        target_id: int = 0,
        source_cls_str: str = '',
        source_id: int = 0):
    content_model = apps.get_model(content_cls_str)
    content = content_model.objects.get(pk=content_id)
    content_ct = ContentType.objects.get_for_model(content)

    target = None
    if target_cls_str != '':
        model = apps.get_model(target_cls_str)
        target = model.objects.get(pk=target_id)
    source = None
    if target_cls_str != '':
        model = apps.get_model(source_cls_str)
        source = model.objects.get(pk=source_id)

    notification = django_vox.models.Notification.objects.get(
        codename=codename, content_type=content_ct)
    notification.issue(content, target, source)


try:
    from background_task import background
    issue_notification = background(queue='django-vox')(issue_notification)
except ImportError:
    pass
except RuntimeError:
    pass



import json
from unittest.mock import MagicMock, patch

import django.conf
from django.test import TestCase

import django_vox.backends.postmark_email
import django_vox.backends.template_email
import django_vox.backends.twilio
from django_vox import base


def mocked_requests_post(url, _data=None, json=None, **_kwargs):
    class MockResponse:
        def __init__(self, json_data, status_code):
            self.json_data = json_data
            self.status_code = status_code

        def json(self):
            return self.json_data

        @property
        def ok(self):
            return self.status_code // 100 == 2

    if url == django_vox.backends.postmark_email.Backend.ENDPOINT:
        if not (json.get('TemplateAlias') or json.get('TemplateId')):
            return MockResponse(
                {'ErrorCode': "403", 'Message': 'details'}, 422)
        else:
            return MockResponse({
                "To": "george@example.org",
                "SubmittedAt": "2014-02-17T07:25:01.4178645-05:00",
                "MessageID": "0a129aee-e1cd-480d-b08d-4f48548ff48d",
                "ErrorCode": 0, "Message": "OK"}, 200)
    return MockResponse(None, 404)


class TestTwilioBackend(TestCase):

    TEXT = 'Here is a text message \n\n for {{ you }}'
    PARAMS = {'you': 'me'}
    SUBJECT = 'IGNORED'
    MESSAGE = 'Here is a text message \n\n for me'
    PREVIEW = 'Here is a text message <br/><br/> for me'

    @classmethod
    def test_build_message(cls):
        backend = django_vox.backends.twilio.Backend()
        message = backend.build_message(cls.SUBJECT, cls.TEXT, cls.PARAMS)
        assert cls.MESSAGE == message

    @classmethod
    def test_preview_message(cls):
        backend = django_vox.backends.twilio.Backend()
        message = backend.preview_message(cls.SUBJECT, cls.TEXT, cls.PARAMS)
        assert cls.PREVIEW == message

    def test_send_message(self):
        backend = django_vox.backends.twilio.Backend()
        message = backend.build_message(self.SUBJECT, self.TEXT, self.PARAMS)
        contact = base.Contact('George', 'sms', '+123')
        with patch('twilio.rest.Client'):
            with self.assertRaises(django.conf.ImproperlyConfigured):
                backend.send_message(contact, message)
            with patch('django.conf.settings'):
                backend.send_message(contact, message)
                import twilio.rest
                client = twilio.rest.Client
                assert len(client.mock_calls) > 1
                assert client.mock_calls[0][0] == ''  # class instantiation
                fname, args, kwargs = client.mock_calls[1]
                assert fname == '().messages.create'
                assert args == ()
                assert len(kwargs) == 3
                assert kwargs['to'] == '+123'
                assert isinstance(kwargs['from_'], MagicMock)
                assert kwargs['body'] == self.MESSAGE


class TestPostmarkBackend(TestCase):

    TEXT = 'line 1 : {{ line_1 }}\n' \
           'line 2 : {{ line_2 }}\n' \
           'line 3 : {{ line_3 }}\n' \
           'line 4 : {{ line_4 }}\n' \
           '\n\n' \
           'c\'est vide'
    PARAMS = {
        'line_1': 'poisson un', 'line_2': 'poisson deux',
        'line_3': 'poisson rouge', 'line_4': 'poisson bleu'}
    SUBJECT = 'SUBJECT'
    MESSAGE = '<html>\n' \
              '<h1>SUBJECT</h1>\n' \
              '<dl>\n' \
              '<dt>line 1</dt><dd>poisson un</dd>\n' \
              '<dt>line 2</dt><dd>poisson deux</dd>\n' \
              '<dt>line 3</dt><dd>poisson rouge</dd>\n' \
              '<dt>line 4</dt><dd>poisson bleu</dd>\n' \
              '<dt>c&#39;est vide</dt><dd></dd>\n' \
              '</dl>\n' \
              '</html>'
    PREVIEW = MESSAGE

    @classmethod
    def test_build_message(cls):
        backend = django_vox.backends.postmark_email.Backend()
        message = backend.build_message(cls.SUBJECT, cls.TEXT, cls.PARAMS)
        assert cls.MESSAGE == message

    @classmethod
    def test_preview_message(cls):
        backend = django_vox.backends.postmark_email.Backend()
        message = backend.preview_message(cls.SUBJECT, cls.TEXT, cls.PARAMS)
        assert cls.PREVIEW == message

    def test_send_message(self):
        backend = django_vox.backends.postmark_email.Backend()
        bad_message = backend.build_message('', self.TEXT, self.PARAMS)
        message = backend.build_message(self.SUBJECT, self.TEXT, self.PARAMS)
        contact = base.Contact('George', 'email', 'george@example.org')
        with patch('requests.post', side_effect=mocked_requests_post):
            with self.assertRaises(django.conf.ImproperlyConfigured):
                backend.send_message(contact, message)
            with self.settings(POSTMARK_API_TOKEN='token'):
                with self.assertRaises(RuntimeError):
                    backend.send_message(contact, bad_message)
                backend.send_message(contact, message)
                import requests
                assert requests.post.mock_calls[1][2]['json'][
                    'TemplateModel']['line 1'] == 'poisson un'


class TestTemplateEmailBackend(TestCase):

    TEXT = '{% block text_body%}' \
           'Here is a message \n\n for {{ you }}' \
           '{% endblock %}' \
           '{% block html_body %}' \
           '<p>Here is a message <br/><br/> for {{ you }}' \
           '{% endblock %}' \
                   ''
    PARAMS = {'you': 'me'}
    SUBJECT = 'SUBJECT'
    MESSAGE = {'subject': 'SUBJECT',
               'text': 'Here is a message \n\n for me',
               'html': '<p>Here is a message <br/><br/> for me'}
    PREVIEW = '<p>Here is a message <br/><br/> for me'

    @classmethod
    def test_build_message(cls):
        backend = django_vox.backends.template_email.Backend()
        message = backend.build_message(cls.SUBJECT, cls.TEXT, cls.PARAMS)
        obj = json.loads(message)
        assert cls.MESSAGE == obj

    @classmethod
    def test_preview_message(cls):
        backend = django_vox.backends.template_email.Backend()
        message = backend.preview_message(cls.SUBJECT, cls.TEXT, cls.PARAMS)
        assert cls.PREVIEW == message



import django.conf
import django.core.mail.backends.base
import django.core.mail.backends.smtp
import django.template
import django.utils.html
import requests
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _

from . import json_webhook

__all__ = ('Backend',)


class Backend(json_webhook.Backend):

    ID = 'postmark-template'
    PROTOCOL = 'email'
    VERBOSE_NAME = _('Email (Postmark Templates)')
    ENDPOINT = 'https://api.postmarkapp.com/email/withTemplate'

    @classmethod
    def send_message(cls, contact, message):
        from_email = django.conf.settings.DEFAULT_FROM_EMAIL
        subject, model = cls.extract_model(message)
        data = {
            'TemplateAlias': subject,
            'TemplateModel': model,
            'From': from_email,
            'To': contact.address,
        }
        token = getattr(django.conf.settings, 'POSTMARK_API_TOKEN', None)
        if token is None:
            raise ImproperlyConfigured(
                'Please set POSTMARK_API_TOKEN in your settings')
        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'X-Postmark-Server-Token': token,
        }

        response = requests.post(cls.ENDPOINT, json=data, headers=headers)
        data_result = response.json()
        if not response.ok:
            raise RuntimeError('Postmark error: {} {}'.format(
                data_result['ErrorCode'], data_result['Message']))



from django.test import TestCase

from . import models


class VariableTests(TestCase):
    """Test the variables that are used in admin"""

    fixtures = ['test']

    @staticmethod
    def test_variables():
        notification = models.Article.get_notification('created')
        variables = notification.get_recipient_variables()
        assert variables.keys() == {'si', 'c:sub', 'c:author', '_static'}
        # check recipient variables first:
        for key in ('si', 'c:sub', 'c:author'):
            assert variables[key]['value'] == 'recipient'
        static = variables['_static']
        assert len(static) == 1
        assert static[0]['label'] == 'Article'
        assert static[0]['value'] == 'content'



import json

import django.conf
import django.core.mail.backends.base
import django.core.mail.backends.smtp
import django.template
import django.utils.html
from django.core import mail
from django.utils.translation import ugettext_lazy as _

from . import base


class MultipartMessage:

    def __init__(self):
        self.subject = ''
        self.text = ''
        self.html = ''

    @classmethod
    def from_dict(cls, obj):
        result = cls()
        result.subject = obj.get('subject')
        result.text = obj.get('text')
        result.html = obj.get('html')
        return result

    @classmethod
    def from_string(cls, string):
        return cls.from_dict(json.loads(string))

    def to_dict(self):
        return {'subject': self.subject, 'text': self.text, 'html': self.html}

    def __str__(self):
        return json.dumps(self.to_dict())

    def to_mail(self) -> mail.EmailMultiAlternatives:
        email = mail.EmailMultiAlternatives()
        email.subject = self.subject
        if self.text:
            email.body = self.text
            if self.html:
                email.attach_alternative(self.html, 'text/html')
        elif self.html:
            email.body = self.html
            email.content_subtype = 'html'
        return email


class Backend(base.Backend):

    ID = 'email-basic'
    PROTOCOL = 'email'
    VERBOSE_NAME = _('Email (Basic)')
    USE_SUBJECT = True

    @classmethod
    def build_multipart(cls, subject: str, body: str,
                        parameters: dict) -> MultipartMessage:
        raise NotImplementedError()

    @classmethod
    def build_message(cls, subject: str, body: str, parameters: dict):
        return str(cls.build_multipart(subject, body, parameters))

    @classmethod
    def preview_message(cls, subject: str, body: str, parameters: dict):
        parts = cls.build_multipart(subject, body, parameters)
        if parts.html:
            return parts.html
        return django.utils.html.escape(parts.text)

    @classmethod
    def send_message(cls, contact, message):
        mpm = MultipartMessage.from_string(message)
        email = mpm.to_mail()
        email.from_email = django.conf.settings.DEFAULT_FROM_EMAIL
        email.to = [contact.address]
        connection = django.core.mail.get_connection()
        connection.send_messages([email])



import django.template
from django.utils.translation import ugettext_lazy as _

from . import base, base_email

__all__ = ('Backend',)


class Backend(base_email.Backend):

    ID = 'email-html'
    VERBOSE_NAME = _('Email (HTML)')
    DEPENDS = ('august',)

    @classmethod
    def build_multipart(cls, subject: str, body: str, parameters: dict):
        return email_html(subject, body, parameters)


def email_html(subject: str, body: str, parameters: dict) -> \
        base_email.MultipartMessage:
    import august
    text_context = django.template.Context(parameters, autoescape=False)
    html_context = django.template.Context(parameters, autoescape=True)
    message = base_email.MultipartMessage()
    # subject
    subject_template = base.template_from_string(subject)
    message.subject = subject_template.render(text_context)
    # html
    body_template = base.template_from_string(body)
    message.html = body_template.render(html_context)
    message.text = august.convert(message.html)
    return message



import datetime
import decimal
import re

import django.template
from django.utils.translation import ugettext_lazy as _

from django_vox import settings

from . import base, base_email

__all__ = ('Backend',)


class Backend(base_email.Backend):

    ID = 'email-md'
    VERBOSE_NAME = _('Email (Markdown)')
    DEPENDS = ('markdown2',)

    @classmethod
    def build_multipart(cls, subject: str, body: str, parameters: dict):
        return email_md(subject, body, parameters)


class MarkdownParameters:

    IGNORED_TYPES = (int, float, decimal.Decimal, datetime.timedelta,
                     datetime.datetime, datetime.date, datetime.time)
    MD_SPECIAL_PATTERN = re.compile(r"[\\`*_{\}\[\]()#+\-.!]")

    @classmethod
    def wrap(cls, obj):
        if callable(obj):
            return CallableMarkdownParameters(obj)
        elif isinstance(obj, cls.IGNORED_TYPES):
            return obj
        return MarkdownParameters(obj)

    def __init__(self, obj):
        self._obj = obj

    def __contains__(self, item):
        return self._obj.__contains__(item)

    def __getitem__(self, item):
        return self.wrap(self._obj[item])

    def __getattr__(self, attr):
        return self.wrap(getattr(self._obj, attr))

    def __str__(self):
        return self.MD_SPECIAL_PATTERN.sub(self.escape, str(self._obj))

    @staticmethod
    def escape(patterns) -> str:
        return '\\' + patterns[0]


class CallableMarkdownParameters(MarkdownParameters):

    def __call__(self, *args, **kwargs):
        obj = self._obj.__call__(*args, **kwargs)
        if callable(obj):
            return CallableMarkdownParameters(obj)
        return MarkdownParameters(obj)


def email_md(subject: str, body: str, parameters: dict) -> \
        base_email.MultipartMessage:
    import markdown2
    md_parameters = MarkdownParameters(parameters)
    text_context = django.template.Context(parameters, autoescape=False)
    md_context = django.template.Context(md_parameters, autoescape=False)
    message = base_email.MultipartMessage()
    subject_template = base.template_from_string(subject)
    body_template = base.template_from_string(body)
    message.subject = subject_template.render(text_context)
    message.text = body_template.render(text_context)
    md_body = body_template.render(md_context)
    message.html = markdown2.markdown(
        md_body, extras=settings.MARKDOWN_EXTRAS,
        link_patterns=settings.MARKDOWN_LINK_PATTERNS)
    return message



"""
Django settings for api documentation generation
"""
import os
import sys
sys.path.insert(0, os.getcwd())
sys.path.insert(0, os.path.join(os.getcwd(), os.pardir))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    "default": {
        "NAME": ":memory:",
        "ENGINE": "django.db.backends.sqlite3",
        "USER": '',
        "PASSWORD": '',
        "PORT": '',
    }
}

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django_vox',
]

SECRET_KEY = 'test'
